@@ -3,10 +3,10 @@
 
 ;; Author: Tsukamoto Tetsuo <czkmt@remus.dti.ne.jp>
 ;; Maintainer: Mikio Nakajima <minakaji@osaka.email.ne.jp>
-;; Version: $Id: skk-jisx0201.el,v 1.3 1999/11/07 02:53:06 minakaji Exp $
+;; Version: $Id: skk-jisx0201.el,v 1.5 2000/09/14 08:39:08 akiho Exp $
 ;; Keywords: japanese
 ;; Created: Oct. 30, 1999.
-;; Last Modified: $Date: 1999/11/07 02:53:06 $
+;; Last Modified: $Date: 2000/09/14 08:39:08 $
 
 ;; This file is not part of SKK yet.
 
@@ -159,7 +159,7 @@ skk-use-color-cursor $B$,(B non-nil $B$N$H$-$K;HMQ$5$l$k!#(B"
 (defvar skk-jisx0201-rule-list
   '(
     ("!" nil "!")
-    ("\"" nil "(J"(B")
+    ("\"" nil "\"")
     ("#" nil "#")
     ;;("$" nil "$")
     ("%" nil "%")
@@ -173,85 +173,85 @@ skk-use-color-cursor $B$,(B non-nil $B$N$H$-$K;HMQ$5$l$k!#(B"
     ("-" nil "-")
     ;;("." nil ".")
     ;;("/" nil "/")
-    ;;("0" nil "(J0(B")
-    ("1" nil "(J1(B")
-    ("2" nil "(J2(B")
-    ("3" nil "(J3(B")
-    ("4" nil "(J4(B")
-    ("5" nil "(J5(B")
-    ("6" nil "(J6(B")
-    ("7" nil "(J7(B")
-    ("8" nil "(J8(B")
-    ("9" nil "(J9(B")
-    (":" nil "(J:(B")
-    (";" nil "(J;(B")
-    ("<" nil "(J<(B")
-    ("=" nil "(J=(B")
-    (">" nil "(J>(B")
-    ("?" nil "(J?(B")
-    ("@" nil "(J@(B")
-    ("A" nil "(JA(B")
-    ("B" nil "(JB(B")
-    ("C" nil "(JC(B")
-    ("D" nil "(JD(B")
-    ("E" nil "(JE(B")
-    ("F" nil "(JF(B")
-    ("G" nil "(JG(B")
-    ("H" nil "(JH(B")
-    ("I" nil "(JI(B")
-    ("J" nil "(JJ(B")
-    ("K" nil "(JK(B")
-    ("L" nil "(JL(B")
-    ("M" nil "(JM(B")
-    ("N" nil "(JN(B")
-    ("O" nil "(JO(B")
-    ("P" nil "(JP(B")
-    ("Q" nil "(JQ(B")
-    ("R" nil "(JR(B")
-    ("S" nil "(JS(B")
-    ("T" nil "(JT(B")
-    ("U" nil "(JU(B")
-    ("V" nil "(JV(B")
-    ("W" nil "(JW(B")
-    ("X" nil "(JX(B")
-    ("Y" nil "(JY(B")
-    ("Z" nil "(JZ(B")
-    ;;("[" nil "(J[(B")
-    ;;("\\" nil "(J\(B")
-    ;;("]" nil "(J](B")
-    ("^" nil "(J^(B")
-    ("_" nil "(J_(B")
-    ("`" nil "(J`(B")
-    ;;("a" nil "(Ja(B")
-    ;;("b" nil "(Jb(B")
-    ;;("c" nil "(Jc(B")
-    ;;("d" nil "(Jd(B")
-    ;;("e" nil "(Je(B")
-    ;;("f" nil "(Jf(B")
-    ;;("g" nil "(Jg(B")
-    ;;("h" nil "(Jh(B")
-    ;;("i" nil "(Ji(B")
-    ;;("j" nil "(Jj(B")
-    ;;("k" nil "(Jk(B")
-    ;;("l" nil "(Jl(B")
-    ;;("m" nil "(Jm(B")
-    ;;("n" nil "(Jn(B")
-    ;;("o" nil "(Jo(B")
-    ;;("p" nil "(Jp(B")
-    ;;("q" nil "(Jq(B")
-    ;;("r" nil "(Jr(B")
-    ;;("s" nil "(Js(B")
-    ;;("t" nil "(Jt(B")
-    ;;("u" nil "(Ju(B")
-    ;;("v" nil "(Jv(B")
-    ;;("w" nil "(Jw(B")
-    ;;("x" nil "(Jx(B")
-    ;;("y" nil "(Jy(B")
-    ;;("z" nil "(Jz(B")
-    ("{" nil "(J{(B")
-    ("|" nil "(J|(B")
-    ("}" nil "(J}(B")
-    ("~" nil "(J~(B")
+    ;;("0" nil "0")
+    ("1" nil "1")
+    ("2" nil "2")
+    ("3" nil "3")
+    ("4" nil "4")
+    ("5" nil "5")
+    ("6" nil "6")
+    ("7" nil "7")
+    ("8" nil "8")
+    ("9" nil "9")
+    (":" nil ":")
+    (";" nil ";")
+    ("<" nil "<")
+    ("=" nil "=")
+    (">" nil ">")
+    ("?" nil "?")
+    ("@" nil "@")
+    ("A" nil "A")
+    ("B" nil "B")
+    ("C" nil "C")
+    ("D" nil "D")
+    ("E" nil "E")
+    ("F" nil "F")
+    ("G" nil "G")
+    ("H" nil "H")
+    ("I" nil "I")
+    ("J" nil "J")
+    ("K" nil "K")
+    ("L" nil "L")
+    ("M" nil "M")
+    ("N" nil "N")
+    ("O" nil "O")
+    ("P" nil "P")
+    ("Q" nil "Q")
+    ("R" nil "R")
+    ("S" nil "S")
+    ("T" nil "T")
+    ("U" nil "U")
+    ("V" nil "V")
+    ("W" nil "W")
+    ("X" nil "X")
+    ("Y" nil "Y")
+    ("Z" nil "Z")
+    ;;("[" nil "[")
+    ;;("\\" nil "\")
+    ;;("]" nil "]")
+    ("^" nil "^")
+    ("_" nil "_")
+    ("`" nil "`")
+    ;;("a" nil "a")
+    ;;("b" nil "b")
+    ;;("c" nil "c")
+    ;;("d" nil "d")
+    ;;("e" nil "e")
+    ;;("f" nil "f")
+    ;;("g" nil "g")
+    ;;("h" nil "h")
+    ;;("i" nil "i")
+    ;;("j" nil "j")
+    ;;("k" nil "k")
+    ;;("l" nil "l")
+    ;;("m" nil "m")
+    ;;("n" nil "n")
+    ;;("o" nil "o")
+    ;;("p" nil "p")
+    ;;("q" nil "q")
+    ;;("r" nil "r")
+    ;;("s" nil "s")
+    ;;("t" nil "t")
+    ;;("u" nil "u")
+    ;;("v" nil "v")
+    ;;("w" nil "w")
+    ;;("x" nil "x")
+    ;;("y" nil "y")
+    ;;("z" nil "z")
+    ("{" nil "{")
+    ("|" nil "|")
+    ("}" nil "}")
+    ("~" nil "~")
     )
 "*SKK JISX0201 $B%b!<%I$NDI2C$N%k!<%k!#(B")
 
@@ -300,7 +300,8 @@ skk-rom-kana-rule-list $B$+$iLZ$N7A$K%3%s%Q%$%k$5$l$k!#(B" )
         skk-jisx0208-latin-mode nil
         skk-katakana nil
         skk-input-mode-string skk-jisx0201-mode-string )
-  (skk-set-cursor-color skk-jisx0201-cursor-color)
+  (and (featurep 'skk-cursor)
+       (skk-set-cursor-color skk-jisx0201-cursor-color)) 
   (force-mode-line-update) )
 
 ;; advices.
@@ -612,7 +613,8 @@ skk-rom-kana-rule-list $B$+$iLZ$N7A$K%3%s%Q%$%k$5$l$k!#(B" )
   (skk-hiragana-to-jisx0201-region start end vcontract)
   (skk-katakana-to-jisx0201-region start end vcontract)
   (set-marker end nil)
-  (skk-set-cursor-properly) )
+  (and (featurep 'skk-cursor)
+       (skk-set-cursor-properly)) )
 
 ;; skk-jisx0201- prefix may be changed to skk-.
 (defun skk-jisx0201-search-and-replace (start end regexp func)
@@ -669,13 +671,23 @@ skk-rom-kana-rule-list $B$+$iLZ$N7A$K%3%s%Q%$%k$5$l$k!#(B" )
 			  (strs (cdr el)))
 		      (if (string= (symbol-value sym) (cdr strs))
 			  (set sym (car strs)) ))))
-		 '((skk-latin-mode-string . ("--SKK:" . " SKK"))
-		   (skk-hiragana-mode-string . ("--$B$+$J(B:" . " $B$+$J(B"))
-		   (skk-katakana-mode-string . ("--$B%+%J(B:" . " $B%+%J(B"))
-		   (skk-jisx0208-latin-mode-string . ("--$BA41Q(B:" . " $BA41Q(B"))
-		   (skk-abbrev-mode-string . ("--a$B$"(B:" . " a$B$"(B"))
-		   (skk-jisx0201-mode-string . ("--jisx0201" . " jisx0201")) ))
-	 (cond ((featurep 'xemacs)
+		 (cond
+		  ((and (fboundp 'face-proportional-p)
+			(face-proportional-p 'modeline))
+		   '((skk-latin-mode-string . ("--SKK:" . " SKK"))
+		     (skk-hiragana-mode-string . ("--$B$+$J(B:" . " $B$+$J(B"))
+		     (skk-katakana-mode-string . ("--$B%+%J(B:" . " $B%+%J(B"))
+		     (skk-jisx0208-latin-mode-string . ("--$BA41Q(B:" . " $BA41Q(B"))
+		     (skk-abbrev-mode-string . ("--a$B$"(B:" . " a$B$"(B"))
+		     (skk-jisx0201-mode-string . ("--jisx0201" . " jisx0201")) ))
+		  (t
+		   '((skk-latin-mode-string . ("--SKK::" . " SKK"))
+		     (skk-hiragana-mode-string . ("--$B$+$J(B:" . " $B$+$J(B"))
+		     (skk-katakana-mode-string . ("--$B%+%J(B:" . " $B%+%J(B"))
+		     (skk-jisx0208-latin-mode-string . ("--$BA41Q(B:" . " $BA41Q(B"))
+		     (skk-abbrev-mode-string . ("--a$B$"(B::" . " a$B$"(B"))
+		     (skk-jisx0201-mode-string . ("--jisx0201" . " jisx0201")) ))))
+	 (cond ((eq skk-emacs-type 'xemacs)
 		(or (memq 'skk-input-mode-string default-mode-line-format)
 		    (setq-default default-modeline-format
 				  (append '("" skk-input-mode-string)
