@@ -5,9 +5,9 @@
 ;; Maintainer: Hideki Sakurada <sakurada@kuis.kyoto-u.ac.jp>
 ;;             Murata Shuuichirou  <mrt@astec.co.jp>
 ;;             Mikio Nakajima <minakaji@osaka.email.ne.jp>
-;; Version: $Id: skk-foreword.el,v 1.14 1999/11/10 12:09:03 minakaji Exp $
+;; Version: $Id: skk-foreword.el,v 1.16 2000/10/12 09:55:27 czkmt Exp $
 ;; Keywords: japanese
-;; Last Modified: $Date: 1999/11/10 12:09:03 $
+;; Last Modified: $Date: 2000/10/12 09:55:27 $
 
 ;; This file is not part of SKK yet.
 
@@ -88,7 +88,7 @@
 
 ;;;###autoload
 (eval-and-compile
-  (defconst skk-emacs-type (cond ((string-match "XEmacs" emacs-version) 'xemacs)
+  (defconst skk-emacs-type (cond ((featurep 'xemacs) 'xemacs)
 				 ((and (boundp 'mule-version)
 				       (string< "4.0" mule-version) 'mule4 ))
 				 ((and (boundp 'mule-version)
@@ -96,20 +96,6 @@
 				 ((and (boundp 'mule-version)
 				       (string< "2.0" mule-version) 'mule2 )))))
 
-(defconst skk-package-data-directory
-  (if (boundp 'early-packages)
-      (let ((dirs (append (if early-package-load-path early-packages)
-			  (if late-package-load-path late-packages)
-			  (if last-package-load-path last-packages) ))
-	    dir )
-	(while (not (file-exists-p (setq dir (car dirs))))
-	  (setq dirs (cdr dirs)) )
-	(and dir
-	     (expand-file-name "skk" (expand-file-name "etc" dir)) ))))
-
-;;(princ (format "SKK-PACKAGE-DATA-DIRECTORY is %s\n"
-;;	       skk-package-data-directory ))
-
 ;; necessary macro and functions to be declared before user variable declarations.
 
 ;;;; macros
