@@ -1,13 +1,13 @@
 ; SKK tutorial for SKK version 10.46 and later versions
 ;; Copyright (C) 1988, 1989, 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997,
-;;               1998, 1999
+;;               1998, 1999, 2000
 ;; Masahiko Sato <masahiko@kuis.kyoto-u.ac.jp>
 
 ;; Author: Masahiko Sato <masahiko@kuis.kyoto-u.ac.jp>
 ;; Maintainer: Mikio Nakajima <minakaji@osaka.email.ne.jp>
-;; Version: $Id: skk-tut.el,v 1.9 1999/10/03 11:51:49 minakaji Exp $
+;; Version: $Id: skk-tut.el,v 1.13 2000/10/18 15:47:51 czkmt Exp $
 ;; Keywords: japanese
-;; Last Modified: $Date: 1999/10/03 11:51:49 $
+;; Last Modified: $Date: 2000/10/18 15:47:51 $
 
 ;; This file is part of SKK.
 
@@ -37,11 +37,8 @@
 
 ;; User variables.  prefix should be `skk-tut-'.
 (defcustom skk-tut-file 
-  (cond
-   ((and skk-package-data-directory
-	 (file-exists-p (expand-file-name "SKK.tut" skk-package-data-directory)) )
-    (expand-file-name "SKK.tut" skk-package-data-directory) )
-   (t "/usr/local/share/skk/SKK.tut") )
+  (static-cond ((eq skk-emacs-type 'xemacs) (locate-data-file "SKK.tut"))
+	       (t "/usr/local/share/skk/SKK.tut"))
   "*SKK $B%A%e!<%H%j%"%k$N%U%!%$%kL>!#(B
 The English version is SKK.tut.E."
   :type 'file
@@ -369,28 +366,30 @@ The English version is SKK.tut.E."
 				 global-map)
       (substitute-key-definition 'canna-self-insert-command 'skk-insert map
 				 global-map)
+      (substitute-key-definition 'canna-henkan-region-or-self-insert
+				 'skk-insert map global-map)
       (substitute-key-definition 'can-n-egg-self-insert-command 'skk-insert map
 				 global-map)
       (define-key map "x" 'skk-previous-candidate)
       (define-key map "\C-j" 'skk-kakutei)
       (define-key map "\t" 'skk-insert)
-      (setq skktut-j-mode-map map) ))
+      (setq skktut-j-mode-map map)))
 
 (defvar skktut-jisx0208-latin-mode-map nil
   "SKK $B%A%e!<%H%j%"%kA43Q1Q?t;z%b!<%I%-!<%^%C%W!#(B" )
 
 (or skktut-jisx0208-latin-mode-map
     (let ((map (make-sparse-keymap))
-	  (i 0) )
+	  (i 0))
       (while (< i 128)
 	(if (aref skk-jisx0208-latin-vector i)
-	    (define-key map (char-to-string i) 'skk-jisx0208-latin-insert) )
-	(setq i (1+ i)) )
+	    (define-key map (char-to-string i) 'skk-jisx0208-latin-insert))
+	(setq i (1+ i)))
       (define-key map "\C-j" 'skk-kakutei)
-      (setq skktut-jisx0208-latin-mode-map map) ))
+      (setq skktut-jisx0208-latin-mode-map map)))
 
 (defvar skktut-abbrev-mode-map nil
-  "SKK $B%A%e!<%H%j%"%k(B Abbrev $B%b!<%I%-!<%^%C%W!#(B" )
+  "SKK $B%A%e!<%H%j%"%k(B Abbrev $B%b!<%I%-!<%^%C%W!#(B")
 
 (or skktut-abbrev-mode-map
     (let ((map (make-sparse-keymap)))
@@ -400,7 +399,7 @@ The English version is SKK.tut.E."
       (define-key map "\C-j" 'skk-kakutei)
       (define-key map " " 'skk-start-henkan)
       (define-key map "\t" 'skk-try-completion)
-      (setq skktut-abbrev-mode-map map) ))
+      (setq skktut-abbrev-mode-map map)))
 
 ;; -- macros
 (defmacro skktut-message (japanese english &rest arg)
@@ -445,8 +444,10 @@ The English version is SKK.tut.E."
        (= skktut-question-count 1)
        (skktut-error "$B$3$N%-!<$O$^$@;H$($^$;$s(B" "Cannot use this key yet" ) ))
 
+;;; XXX interactive subr with args.
 (defadvice kill-buffer (around skktut-ad disable)
   "SKK $B%A%e!<%H%j%"%kMQ%"%I%P%$%9IU!#(B"
+  (interactive "bKill buffer: ")
   (cond ((or (not (interactive-p))
 	     (null (member (ad-get-arg 0) (list skktut-working-buffer
 						skktut-question-buffer
@@ -928,27 +929,27 @@ C-u M-x skk-tutorial-quit $B$9$k$H!"(Byes-or-no-p $B$G?R$M$i$l$k$3$H$J$/D>$A$
      (if skktut-japanese-tut
 	 (concat "SKK $B%A%e!<%H%j%"%k$O$3$l$G=*$j$G$9!#(B\n\n"
 		 "SKK 10.x $B$K4X$9$k<ALd!"%3%a%s%H!"(Bbug report $BEy$O(B\n\n"
-		 "\tskk-develop@kuis.kyoto-u.ac.jp\n\n"
-		 "$BKx$*Aw$j2<$5$$!#$J$*!"$3$N%"%I%l%9$O3+H/<TMQ(B SKK $B%a%$%j%s%0(B"
-		 "$B%j%9%H$N%"%I%l%9$G$9!#(B\n"
+		 "\tskk@ring.gr.jp\n\n"
+		 "$BKx$*Aw$j2<$5$$!#$J$*!"$3$N%"%I%l%9$O(B SKK Ring Server Openlab $B%a%$%j%s%0(B"
+ 		 "$B%j%9%H$N%"%I%l%9$G$9!#(B\n"
 		 "$B2sEz$ODL>o$3$N%"%I%l%9$KBP$7$F$J$5$l$k$N$G!"%a%s%P!<$G$J$$(B"
 		 "$BJ}$O$=$N;]$rL@5-$7$F(B\n"
-		 "$B%a!<%k$r$*Aw$j$/$@$5$$!#3+H/<TMQ(B SKK $B%a%$%j%s%0%j%9%H$X;2(B"
+		 "$B%a!<%k$r$*Aw$j$/$@$5$$!#(BSKK Ring Server Openlab $B%a%$%j%s%0%j%9%H$X;2(B"
 		 "$B2C4uK>$N>l9g$O(B\n\n"
-		 "\tskk-develop-join@kuis.kyoto-u.ac.jp\n\n"
+		 "\tskk-request@ring.gr.jp\n\n"
 		 "$B$X%a!<%k$r$*Aw$j$/$@$5$$(B\n\n"
 		 "!! $B:G8e$K(B <return> $B%-!<$r2!$7$F$/$@$5$$!#(B" )
        (concat "Now we end the SKK tutorial.\n\n"
 	       "Please send comments, questions and bug reports on SKK "
 	       "version 10.x to:\n\n"
-	       "\tskk-develop@kuis.kyoto-u.ac.jp\n\n"
-	       "This is the address of the SKK developers mailing list, and "
+	       "\tskk@ring.gr.jp\n\n"
+	       "This is the address of the SKK Ring Server Openlab mailing list, and "
 	       "normally the responces\n"
 	       "will be sent only to the ML members.  So, if you are not a ML "
 	       "member, please say so \n"
-	       "in your mail. If you are interested in joining the SKK "
-	       "developers ML, send a mail to:\n\n"
-	       "\tskk-develop-join@kuis.kyoto-u.ac.jp\n\n"
+	       "in your mail. If you are interested in joining the SKK Ring Server "
+	       "Openlab ML, send a mail to:\n\n"
+	       "\tskk-request@ring.gr.jp\n\n"
 	       "!! Hit <return> key when you are ready." )))
     (if skk-tut-use-face
 	(save-match-data
